#! /usr/bin/perl

use strict;
use warnings FATAL => 'all';

use Time::Local;

my $NODE_TYPE = 0;
my $NODE_DATE = 1;

my $LINK_NODE1 = 0;
my $LINK_NODE2 = 1;
my $LINK_TYPE = 2;

my %nodes = ();
my %links = ();
my %author_hash = ();
while( my $line = <STDIN> ) {
	chomp( $line );
	if( $line =~ /<OBJECT ID=\"(\d+)\"\/>/ ) {
		my $node_id = $1;
		if( !defined( $nodes{$node_id} ) ) {
			$nodes{$node_id} = [];
		}
	} elsif( $line =~ /<LINK ID=\"(\d+)\" O1-ID=\"(\d+)\" O2-ID=\"(\d+)\"\/>/ ) {
		my $link_id = $1;
		my $node1_id = $2;
		my $node2_id = $3;
		if( !defined( $links{$link_id} ) ) {
			$links{$link_id} = [];
		}
		$links{$link_id}->[$LINK_NODE1] = $node1_id;
		$links{$link_id}->[$LINK_NODE2] = $node2_id;
	} elsif( $line =~ /<ATTRIBUTE NAME=\"objecttype\".*/ ) {
		while( my $array_ref = read_attribute() ) {
			my $id = $array_ref->[0];
			my $value = $array_ref->[1];
			$nodes{$id}->[$NODE_TYPE] = $value;
		}
	} elsif( $line =~ /<ATTRIBUTE NAME=\"earliest_date\".*/ ) {
		while( my $array_ref = read_attribute() ) {
			my $id = $array_ref->[0];
			my $value = $array_ref->[1];
			$nodes{$id}->[$NODE_DATE] = $value;
		}
	} elsif( $line =~ /<ATTRIBUTE NAME=\"linktype\".*/ ) {
		while( my $array_ref = read_attribute() ) {
			my $id = $array_ref->[0];
			my $value = $array_ref->[1];
			$links{$id}->[$LINK_TYPE] = $value;
			
			my $node1_id = $links{$id}->[$LINK_NODE1];
			my $node2_id = $links{$id}->[$LINK_NODE2];
			if( $value eq "Authored" || $value eq "Co_Authored" ) {
				if( !defined( $author_hash{$node2_id} ) ) {
					$author_hash{$node2_id} = [];
				}
				push( @{$author_hash{$node2_id}}, $node1_id );
			}
		}
	}
}

foreach my $paper_id (keys(%author_hash)) {
	my $year = $nodes{$paper_id}->[$NODE_DATE];
	my $author_array_ref = $author_hash{$paper_id};
	foreach my $index1 (0..$#$author_array_ref-1) {
		foreach my $index2 ($index1+1..$#$author_array_ref) {
			my $author1 = $author_array_ref->[$index1];
			my $author2 = $author_array_ref->[$index2];
			# using the max index is correct, since the weight is shared across the number of OTHER authors instead of the number of authors
			if( $author1 != $author2 ) {
				printf STDOUT "$year,$author1,$author2,%.6f\n", 1 / $#$author_array_ref;
				printf STDOUT "$year,$author2,$author1,%.6f\n", 1 / $#$author_array_ref;
			}
		}
	}
}

sub read_attribute {
	my $line = <STDIN>;
	if( $line =~ /<\/ATTRIBUTE>/ ) {
		return 0;
	} else {
		$line =~ /<ATTR-VALUE ITEM-ID=\"(\d+)\">/;
		my $id = $1;
		$line = <STDIN>;
		$line =~ /<COL-VALUE>(.*)<\/COL-VALUE>/;
		my $value = $1;
		$line = <STDIN>;
		return [$id,$value];
	}
}